@@ -6,16 +6,16 @@ FOUNDTEX=0
 BINPATHS=":"
 
 if [ -x /usr/share/libtool/config.guess ]; then
-	PLATFORM=`/usr/share/libtool/config.guess`
-	ARCH=`echo $PLATFORM | sed 's/-.*//;s/i.86/i386/'`
+	PLATFORM=$(/usr/share/libtool/config.guess)
+	ARCH=$(echo $PLATFORM | sed 's/-.*//;s/i.86/i386/')
 # At least on Debian, Ubuntu, Gentoo and LFS, config.guess seems to be in a
 # subdirectory
 elif [ -x /usr/share/libtool/config/config.guess ]; then
-	PLATFORM=`/usr/share/libtool/config/config.guess`
-	ARCH=`echo $PLATFORM | sed 's/-.*//;s/i.86/i386/'`
+	PLATFORM=$(/usr/share/libtool/config/config.guess)
+	ARCH=$(echo $PLATFORM | sed 's/-.*//;s/i.86/i386/')
 else
-	PLATFORM=`uname -s | tr A-Z a-z`
-	ARCH=`uname -m | tr A-Z a-z | sed 's/i.86/i386/'`
+	PLATFORM=$(uname -s | tr A-Z a-z)
+	ARCH=$(uname -m | tr A-Z a-z | sed 's/i.86/i386/')
 fi
 
 # append a path to $BINPATHS unless already present
@@ -42,18 +42,18 @@ esac
 
 # (1) try to find tex and ghostscript
 
-TEX=`which tex`
+TEX=$(which tex)
 if [ "$TEX" != "" ]; then
 	if [ -x "$TEX" ]; then
-		appendPath `dirname "$TEX"`
+		appendPath $(dirname "$TEX")
 		FOUNDTEX=1
 	fi
 fi
 
-GS=`which gs`
+GS=$(which gs)
 if [ "$GS" != "" ]; then
 	if [ -x "$GS" ]; then
-		appendPath `dirname "$GS"`
+		appendPath $(dirname "$GS")
 	fi
 fi
 
@@ -71,9 +71,12 @@ case $PLATFORM in
 	*netbsd*)	OS=netbsd;;
 	*openbsd*)	OS=openbsd;;
 	*solaris*)	OS=solaris;;
-	*)			OS=`echo $PLATFORM | sed 's/.*-//'`
+	*)			OS=$(echo $PLATFORM | sed 's/.*-//')
 esac
 
+appendPath "/usr/local/texlive/2021/bin/$ARCH-$OS"
+appendPath "/usr/local/texlive/2020/bin/$ARCH-$OS"
+appendPath "/usr/local/texlive/2019/bin/$ARCH-$OS"
 appendPath "/usr/local/texlive/2018/bin/$ARCH-$OS"
 appendPath "/usr/local/texlive/2017/bin/$ARCH-$OS"
 appendPath "/usr/local/texlive/2016/bin/$ARCH-$OS"
@@ -88,7 +91,7 @@ for TEXLIVEROOT in /usr/local/texlive/* /opt/texlive/*; do
 	# Check if this is really a folder (e.g., /opt/... might not exist)
 	if [ -d "$TEXLIVEROOT/bin/$ARCH-$OS" ]; then
 		# Check that this is of the form /texlive/1234
-		if [ -z `basename $TEXLIVEROOT | sed 's/[0-9]//g'` ]; then
+		if [ -z $(basename $TEXLIVEROOT | sed 's/[0-9]//g') ]; then
 			# Paranoia: Make sure there actually is a bin/... subdirectory
 			if [ -d "$TEXLIVEROOT/bin/$ARCH-$OS" ]; then
 				appendPath "$TEXLIVEROOT/bin/$ARCH-$OS"
@@ -102,7 +105,7 @@ done
 BINPATHS="$BINPATHS/usr/local/bin:/usr/bin"
 
 # strip leading and trailing colons from the list
-BINPATHS=`echo $BINPATHS | sed 's/^://;s/:$//'`
+BINPATHS=$(echo $BINPATHS | sed 's/^://;s/:$//')
 
 echo "// Default paths to TeX binaries, for TeXworks" > $PATHFILE
 echo "// Generated by $0" >> $PATHFILE
