@@ -1,6 +1,6 @@
 /*
 	This is part of TeXworks, an environment for working with TeX documents
-	Copyright (C) 2007-2015  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
+	Copyright (C) 2007-2019  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -70,7 +70,7 @@ const int kPDFHighlightDuration = 2000;
 QList<PDFDocument*> PDFDocument::docList;
 
 PDFDocument::PDFDocument(const QString &fileName, TeXDocument *texDoc)
-	: _syncHighlight(NULL), _synchronizer(NULL), openedManually(false)
+	: _syncHighlight(NULL), _synchronizer(NULL), openedManually(false), _fullScreenManager(NULL)
 {
 	init();
 
@@ -80,16 +80,16 @@ PDFDocument::PDFDocument(const QString &fileName, TeXDocument *texDoc)
 	loadFile(fileName);
 
 	QMap<QString,QVariant> properties = TWApp::instance()->getFileProperties(curFile);
-	if (properties.contains("geometry"))
-		restoreGeometry(properties.value("geometry").toByteArray());
+	if (properties.contains(QString::fromLatin1("geometry")))
+		restoreGeometry(properties.value(QString::fromLatin1("geometry")).toByteArray());
 	else
 		TWUtils::zoomToHalfScreen(this, true);
 
-	if (properties.contains("state"))
-		restoreState(properties.value("state").toByteArray(), kPDFWindowStateVersion);
+	if (properties.contains(QString::fromLatin1("state")))
+		restoreState(properties.value(QString::fromLatin1("state")).toByteArray(), kPDFWindowStateVersion);
 	
-	if (properties.contains("pdfPageMode"))
-		setPageMode(properties.value("pdfPageMode", -1).toInt());
+	if (properties.contains(QString::fromLatin1("pdfPageMode")))
+		setPageMode(properties.value(QString::fromLatin1("pdfPageMode"), -1).toInt());
 
 	QTimer::singleShot(100, this, SLOT(setDefaultScale()));
 
@@ -122,9 +122,9 @@ void PDFDocument::init()
 #if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
 	// The Compiz window manager doesn't seem to support icons larger than
 	// 128x128, so we add a suitable one first
-	winIcon.addFile(":/images/images/TeXworks-doc-128.png");
+	winIcon.addFile(QString::fromLatin1(":/images/images/TeXworks-doc-128.png"));
 #endif
-	winIcon.addFile(":/images/images/TeXworks-doc.png");
+	winIcon.addFile(QString::fromLatin1(":/images/images/TeXworks-doc.png"));
 	setWindowIcon(winIcon);
 
 	pdfWidget = new QtPDF::PDFDocumentWidget(this);
@@ -136,10 +136,12 @@ void PDFDocument::init()
 
 	connect(pdfWidget, SIGNAL(changedPage(int)), this, SLOT(updateStatusBar()));
 	connect(pdfWidget, SIGNAL(changedZoom(qreal)), this, SLOT(updateStatusBar()));
-	connect(pdfWidget, SIGNAL(changedDocument(const QWeakPointer<QtPDF::Backend::Document>)), this, SLOT(updateStatusBar()));
-	connect(pdfWidget, SIGNAL(changedDocument(const QWeakPointer<QtPDF::Backend::Document>)), this, SLOT(invalidateSyncHighlight()));
+	connect(pdfWidget, SIGNAL(changedDocument(const QWeakPointer<QtPDF::Backend::Document>)), this, SLOT(changedDocument(const QWeakPointer<QtPDF::Backend::Document>)));
 	connect(pdfWidget, SIGNAL(searchResultHighlighted(const int, const QList<QPolygonF>)), this, SLOT(searchResultHighlighted(const int, const QList<QPolygonF>)));
 	connect(pdfWidget, SIGNAL(changedPageMode(QtPDF::PDFDocumentView::PageMode)), this, SLOT(updatePageMode(QtPDF::PDFDocumentView::PageMode)));
+	connect(pdfWidget, SIGNAL(requestOpenPdf(QString,QtPDF::PDFDestination,bool)), this, SLOT(maybeOpenPdf(QString,QtPDF::PDFDestination,bool)));
+	connect(pdfWidget, SIGNAL(requestOpenUrl(QUrl)), this, SLOT(maybeOpenUrl(QUrl)));
+	connect(pdfWidget, SIGNAL(textSelectionChanged(bool)), this, SLOT(maybeEnableCopyCommand(bool)));
 
 	toolButtonGroup = new QButtonGroup(toolBar);
 	toolButtonGroup->addButton(qobject_cast<QAbstractButton*>(toolBar->widgetForAction(actionMagnify)), QtPDF::PDFDocumentView::MouseMode_MagnifyingGlass);
@@ -173,6 +175,8 @@ void PDFDocument::init()
 
 	connect(actionQuit_TeXworks, SIGNAL(triggered()), TWApp::instance(), SLOT(maybeQuit()));
 
+	connect(actionCopy, SIGNAL(triggered(bool)), this, SLOT(copySelectedTextToClipboard()));
+
 	connect(actionFind, SIGNAL(triggered()), this, SLOT(doFindDialog()));
 
 	connect(actionFirst_Page, SIGNAL(triggered()), pdfWidget, SLOT(goFirst()));
@@ -187,6 +191,7 @@ void PDFDocument::init()
 	connect(actionActual_Size, SIGNAL(triggered()), pdfWidget, SLOT(zoom100()));
 	connect(actionFit_to_Width, SIGNAL(triggered()), pdfWidget, SLOT(zoomFitWidth()));
 	connect(actionFit_to_Window, SIGNAL(triggered()), pdfWidget, SLOT(zoomFitWindow()));
+	connect(actionFit_to_Content_Width, SIGNAL(triggered()), pdfWidget, SLOT(zoomFitContentWidth()));
 	connect(actionZoom_In, SIGNAL(triggered()), pdfWidget, SLOT(zoomIn()));
 	connect(actionZoom_Out, SIGNAL(triggered()), pdfWidget, SLOT(zoomOut()));
 	connect(actionFull_Screen, SIGNAL(triggered()), this, SLOT(toggleFullScreen()));
@@ -199,8 +204,8 @@ void PDFDocument::init()
 	connect(actionPageMode_TwoPagesContinuous, SIGNAL(triggered()), &pageModeSignalMapper, SLOT(map()));
 	connect(&pageModeSignalMapper, SIGNAL(mapped(int)), this, SLOT(setPageMode(int)));
 
-	if (actionZoom_In->shortcut() == QKeySequence("Ctrl++"))
-		new QShortcut(QKeySequence("Ctrl+="), pdfWidget, SLOT(zoomIn()));
+	if (actionZoom_In->shortcut() == QKeySequence(tr("Ctrl++")))
+		new QShortcut(QKeySequence(tr("Ctrl+=")), pdfWidget, SLOT(zoomIn()));
 	
 	connect(actionTypeset, SIGNAL(triggered()), this, SLOT(retypeset()));
 	
@@ -260,11 +265,9 @@ void PDFDocument::init()
 	dw->hide();
 	addDockWidget(Qt::LeftDockWidgetArea, dw);
 	menuShow->addAction(dw->toggleViewAction());
-
-	exitFullscreen = NULL;
 	
 	QSETTINGS_OBJECT(settings);
-	switch(settings.value("pdfPageMode", kDefault_PDFPageMode).toInt()) {
+	switch(settings.value(QString::fromLatin1("pdfPageMode"), kDefault_PDFPageMode).toInt()) {
 		case 0:
 			setPageMode(QtPDF::PDFDocumentView::PageMode_SinglePage);
 			break;
@@ -278,11 +281,12 @@ void PDFDocument::init()
 			setPageMode(kDefault_PDFPageMode);
 			break;
 	}
+	resetMagnifier();
 
-	if (settings.contains("previewResolution"))
-		pdfWidget->setResolution(settings.value("previewResolution", QApplication::desktop()->logicalDpiX()).toInt());
+	if (settings.contains(QString::fromLatin1("previewResolution")))
+		pdfWidget->setResolution(settings.value(QString::fromLatin1("previewResolution"), QApplication::desktop()->logicalDpiX()).toInt());
 
-	TWUtils::applyToolbarOptions(this, settings.value("toolBarIconSize", 2).toInt(), settings.value("toolBarShowText", false).toBool());
+	TWUtils::applyToolbarOptions(this, settings.value(QString::fromLatin1("toolBarIconSize"), 2).toInt(), settings.value(QString::fromLatin1("toolBarShowText"), false).toBool());
 
 	TWApp::instance()->updateWindowMenus();
 	
@@ -291,6 +295,12 @@ void PDFDocument::init()
 	
 	TWUtils::insertHelpMenuItems(menuHelp);
 	TWUtils::installCustomShortcuts(this);
+
+	pdfWidget->setMouseTracking(true);
+	_fullScreenManager = new FullscreenManager(this);
+	_fullScreenManager->addShortcut(actionFull_Screen, SLOT(toggleFullScreen()));
+	connect(_fullScreenManager, SIGNAL(fullscreenChanged(bool)), actionFull_Screen, SLOT(setChecked(bool)));
+	connect(_fullScreenManager, SIGNAL(fullscreenChanged(bool)), this, SLOT(maybeZoomToWindow(bool)), Qt::QueuedConnection);
 }
 
 void PDFDocument::changeEvent(QEvent *event)
@@ -343,6 +353,20 @@ void PDFDocument::updateRecentFileActions()
 void PDFDocument::updateWindowMenu()
 {
 	TWUtils::updateWindowMenu(this, menuWindow);
+
+	// If the window list changed, we might want to update our window title as
+	// well to uniquely identify the current file among all others open in
+	// TeXworks
+	Q_FOREACH(QAction * action, menuWindow->actions()) {
+		SelWinAction * selWinAction = qobject_cast<SelWinAction*>(action);
+		// If this is not an action related to an open window, skip it
+		if (!selWinAction)
+			continue;
+		// If this action corresponds to the current file, use it's label as
+		// window text
+		if (selWinAction->data().toString() == fileName())
+			setWindowTitle(tr("%1[*] - %2").arg(selWinAction->text()).arg(tr(TEXWORKS_NAME)));
+	}
 }
 
 void PDFDocument::sideBySide()
@@ -381,10 +405,10 @@ void PDFDocument::closeEvent(QCloseEvent *event)
 void PDFDocument::saveRecentFileInfo()
 {
 	QMap<QString,QVariant> fileProperties;
-	fileProperties.insert("path", curFile);
-	fileProperties.insert("geometry", saveGeometry());
-	fileProperties.insert("state", saveState(kPDFWindowStateVersion));
-	fileProperties.insert("pdfPageMode", pdfWidget->pageMode());
+	fileProperties.insert(QString::fromLatin1("path"), curFile);
+	fileProperties.insert(QString::fromLatin1("geometry"), saveGeometry());
+	fileProperties.insert(QString::fromLatin1("state"), saveState(kPDFWindowStateVersion));
+	fileProperties.insert(QString::fromLatin1("pdfPageMode"), pdfWidget->pageMode());
 	TWApp::instance()->addToRecentFiles(fileProperties);
 }
 
@@ -393,7 +417,7 @@ void PDFDocument::loadFile(const QString &fileName)
 	setCurrentFile(fileName);
 	QSETTINGS_OBJECT(settings);
 	QFileInfo info(fileName);
-	settings.setValue("openDialogDir", info.canonicalPath());
+	settings.setValue(QString::fromLatin1("openDialogDir"), info.canonicalPath());
 
 	reload();
 }
@@ -429,16 +453,37 @@ void PDFDocument::loadSyncData()
 }
 
 void PDFDocument::syncClick(int pageIndex, const QPointF& pos)
+{
+	QSETTINGS_OBJECT(settings);
+	TWSynchronizer::Resolution res;
+	switch (settings.value(QString::fromLatin1("syncResolutionToTeX"), TWSynchronizer::kDefault_Resolution_ToTeX).toInt()) {
+		case 0:
+			res = TWSynchronizer::CharacterResolution;
+			break;
+		case 1:
+			res = TWSynchronizer::WordResolution;
+			break;
+		case 2:
+			res = TWSynchronizer::LineResolution;
+			break;
+		default:
+			res = TWSynchronizer::kDefault_Resolution_ToPDF;
+	}
+
+	syncRange(pageIndex, pos, pos, res);
+}
+
+void PDFDocument::syncRange(const int pageIndex, const QPointF & start, const QPointF & end, const TWSynchronizer::Resolution resolution)
 {
 	if (!_synchronizer)
 		return;
 
 	clearSyncHighlight();
 
-	/* NOTE: PDF coordinates are upside down (i.e., (0,0) is in the lower left),
-	 *       whereas SyncTeX expects TeX coordinates (i.e., (0,0) in the upper
-	 *       left). Hence we need to convert the coordinates
-	 */
+	// NOTE: "start" and "end" are in PDF coordinates, which are upside down
+	// (i.e., (0,0) is in the lower left), whereas SyncTeX expects TeX
+	// coordinates (i.e., (0,0) in the upper left). Hence we need to convert the
+	// coordinates.
 	QSharedPointer<QtPDF::Backend::Document> doc = pdfWidget->document().toStrongRef();
 	if (!doc)
 		return;
@@ -446,24 +491,72 @@ void PDFDocument::syncClick(int pageIndex, const QPointF& pos)
 	if (!page)
 		return;
 
-	TWSynchronizer::PDFSyncPoint src;
-	src.filename = curFile;
-	src.page = pageIndex + 1;
-	src.rects.append(QRectF(pos.x(), page->pageSizeF().height() - pos.y(), 0, 0));
+	// Synchronize the point "start"
+	TWSynchronizer::PDFSyncPoint srcStart;
+	srcStart.filename = curFile;
+	srcStart.page = pageIndex + 1;
+	srcStart.rects.append(QRectF(start.x(), page->pageSizeF().height() - start.y(), 0, 0));
+	TWSynchronizer::TeXSyncPoint destStart = _synchronizer->syncFromPDF(srcStart, resolution);
 
-	// Get target point
-	TWSynchronizer::TeXSyncPoint dest = _synchronizer->syncFromPDF(src);
+	// Syncronize the point "end"
+	TWSynchronizer::TeXSyncPoint destEnd;
+	if (end.isNull() || end == start)
+		// If "end" was not provided or was the same as "start", just copy the
+		// result
+		destEnd = destStart;
+	else {
+		// Otherwise, perform the synchronization
+		TWSynchronizer::PDFSyncPoint srcEnd;
+		srcEnd.filename = curFile;
+		srcEnd.page = pageIndex + 1;
+		srcEnd.rects.append(QRectF(end.x(), page->pageSizeF().height() - end.y(), 0, 0));
+		destEnd = _synchronizer->syncFromPDF(srcEnd, resolution);
+	}
 
-	// Check target point
-	if (dest.filename.isEmpty() || dest.line < 0)
+	// Check if (at least) "start" was properly synchronized; if not: bail out
+	if (destStart.filename.isEmpty() || destStart.line < 0)
 		return;
 
-	// Display the result
+	// Open the destination document (for the point "start"), and put the cursor
+	// on the right line (though not necessarily on the right column or with the
+	// the right selection, yet, as that requires additional handling below)
 	QDir curDir(QFileInfo(curFile).canonicalPath());
-	if (dest.col >= 0)
-		TeXDocument::openDocument(QFileInfo(curDir, dest.filename).canonicalFilePath(), true, true, dest.line, dest.col, dest.col + 1);
-	else
-		TeXDocument::openDocument(QFileInfo(curDir, dest.filename).canonicalFilePath(), true, true, dest.line, -1, -1);
+	TeXDocument * texDoc = TeXDocument::openDocument(QFileInfo(curDir, destStart.filename).canonicalFilePath(), true, true, destStart.line);
+	if (!texDoc)
+		return;
+
+	// Get a text cursor in the correct position for "start" (if no valid column
+	// was found, place it at the beginning of the correct line)
+	QTextCursor curStart = texDoc->textCursor();
+	curStart.setPosition(0);
+	curStart.movePosition(QTextCursor::NextBlock, QTextCursor::MoveAnchor, destStart.line - 1);
+	if (destStart.col >= 0)
+		curStart.movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor, destStart.col);
+
+	// Get a text cursor in the correct position for "end" (if no valid column
+	// was found, place it at the end of the correct line)
+	QTextCursor curEnd = texDoc->textCursor();
+	if (destEnd.filename == destStart.filename) {
+		curEnd.setPosition(0);
+		curEnd.movePosition(QTextCursor::NextBlock, QTextCursor::MoveAnchor, destStart.line - 1);
+		if (destEnd.col >= 0)
+			curEnd.movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor, destEnd.col + qMax(1, destEnd.len));
+		else
+			curEnd.movePosition(QTextCursor::EndOfBlock);
+	}
+	else {
+		// If we get here, the end point is in a different file than the
+		// starting point. In that case, we assume the rest of the file
+		// containing the starting point should be highlighted)
+		curEnd.movePosition(QTextCursor::End);
+	}
+
+	// Properly highlight the destination document
+	// NB: We use TeXDocument::openDocument here even though we already have a
+	// pointer to the document as that is the only publicly available function
+	// that does what we need (i.e., position the cursor and possibly change the
+	// current selection).
+	TeXDocument::openDocument(QFileInfo(curDir, destStart.filename).canonicalFilePath(), true, true, destStart.line, curStart.position() - curStart.block().position(), curEnd.position() - curStart.block().position());
 }
 
 void PDFDocument::syncFromSource(const QString& sourceFile, int lineNo, int col, bool activatePreview)
@@ -471,13 +564,29 @@ void PDFDocument::syncFromSource(const QString& sourceFile, int lineNo, int col,
 	if (!_synchronizer)
 		return;
 
+	QSETTINGS_OBJECT(settings);
+	TWSynchronizer::Resolution res;
+	switch (settings.value(QString::fromLatin1("syncResolutionToPDF"), TWSynchronizer::kDefault_Resolution_ToPDF).toInt()) {
+		case 0:
+			res = TWSynchronizer::CharacterResolution;
+			break;
+		case 1:
+			res = TWSynchronizer::WordResolution;
+			break;
+		case 2:
+			res = TWSynchronizer::LineResolution;
+			break;
+		default:
+			res = TWSynchronizer::kDefault_Resolution_ToPDF;
+	}
+
 	TWSynchronizer::TeXSyncPoint src;
 	src.filename = sourceFile;
 	src.line = lineNo;
 	src.col = col;
 
 	// Get target point
-	TWSynchronizer::PDFSyncPoint dest = _synchronizer->syncFromTeX(src);
+	TWSynchronizer::PDFSyncPoint dest = _synchronizer->syncFromTeX(src, res);
 
 	// Check target point
 	if (dest.page < 1 || QFileInfo(curFile) != QFileInfo(dest.filename))
@@ -531,9 +640,32 @@ void PDFDocument::clearSearchResultHighlight()
 	widget()->setCurrentSearchResultHighlightBrush(QBrush(Qt::transparent));
 }
 
+void PDFDocument::copySelectedTextToClipboard()
+{
+	if (!widget()) return;
+	QString textToCopy = widget()->selectedText();
+	if (textToCopy.isEmpty())
+		return;
+	Q_ASSERT(QApplication::clipboard());
+	QApplication::clipboard()->setText(textToCopy);
+}
+
+void PDFDocument::maybeEnableCopyCommand(const bool isTextSelected)
+{
+  Q_ASSERT(actionCopy);
+  if (!widget())
+	return;
+  QSharedPointer<QtPDF::Backend::Document> doc = widget()->document().toStrongRef();
+  if (!doc)
+	actionCopy->setEnabled(false);
+  else
+	actionCopy->setEnabled(isTextSelected && doc->permissions().testFlag(QtPDF::Backend::Document::Permission_Extract));
+}
+
 void PDFDocument::setCurrentFile(const QString &fileName)
 {
 	curFile = QFileInfo(fileName).canonicalFilePath();
+	//: Format for the window title (ex. "file.pdf[*] - TeXworks")
 	setWindowTitle(tr("%1[*] - %2").arg(TWUtils::strippedName(curFile)).arg(tr(TEXWORKS_NAME)));
 	TWApp::instance()->updateWindowMenus();
 }
@@ -592,6 +724,12 @@ void PDFDocument::goToSource()
 		actionGo_to_Source->setEnabled(false);
 }
 
+void PDFDocument::changedDocument(const QWeakPointer<QtPDF::Backend::Document> newDoc) {
+	updateStatusBar();
+	invalidateSyncHighlight();
+	enablePageActions(pdfWidget->currentPage());
+}
+
 void PDFDocument::enablePageActions(int pageIndex)
 {
 //#if !defined(Q_OS_DARWIN)
@@ -607,23 +745,8 @@ void PDFDocument::enablePageActions(int pageIndex)
 
 void PDFDocument::toggleFullScreen()
 {
-	if (windowState() & Qt::WindowFullScreen) {
-		// exiting full-screen mode
-		statusBar()->show();
-		toolBar->show();
-		showNormal();
-		actionFull_Screen->setChecked(false);
-		delete exitFullscreen;
-	}
-	else {
-		// entering full-screen mode
-		statusBar()->hide();
-		toolBar->hide();
-		showFullScreen();
-		pdfWidget->zoomFitWindow();
-		actionFull_Screen->setChecked(true);
-		exitFullscreen = new QShortcut(Qt::Key_Escape, this, SLOT(toggleFullScreen()));
-	}
+	Q_ASSERT(_fullScreenManager);
+	_fullScreenManager->toggleFullscreen();
 }
 
 void PDFDocument::setPageMode(const int newMode)
@@ -655,15 +778,15 @@ void PDFDocument::resetMagnifier()
 	Q_ASSERT(pdfWidget != NULL);
 	QSETTINGS_OBJECT(settings);
 
-	if (settings.value("circularMagnifier", kDefault_CircularMagnifier).toBool())
+	if (settings.value(QString::fromLatin1("circularMagnifier"), kDefault_CircularMagnifier).toBool())
 		pdfWidget->setMagnifierShape(QtPDF::DocumentTool::MagnifyingGlass::Magnifier_Circle);
 	else
 		pdfWidget->setMagnifierShape(QtPDF::DocumentTool::MagnifyingGlass::Magnifier_Rectangle);
 
-	pdfWidget->setMagnifierSize(magSizes[qBound(0, settings.value("magnifierSize", kDefault_MagnifierSize).toInt() - 1, 2)]);
+	pdfWidget->setMagnifierSize(magSizes[qBound(0, settings.value(QString::fromLatin1("magnifierSize"), kDefault_MagnifierSize).toInt() - 1, 2)]);
 }
 
-void PDFDocument::setResolution(int res)
+void PDFDocument::setResolution(const double res)
 {
 	Q_ASSERT(pdfWidget != NULL);
 	if (res > 0)
@@ -679,14 +802,14 @@ void PDFDocument::updateTypesettingAction(bool processRunning)
 {
 	if (processRunning) {
 		disconnect(actionTypeset, SIGNAL(triggered()), this, SLOT(retypeset()));
-		actionTypeset->setIcon(QIcon(":/images/tango/process-stop.png"));
+		actionTypeset->setIcon(QIcon(QString::fromLatin1(":/images/tango/process-stop.png")));
 		actionTypeset->setText(tr("Abort typesetting"));
 		connect(actionTypeset, SIGNAL(triggered()), this, SLOT(interrupt()));
 		enableTypesetAction(true);
 	}
 	else {
 		disconnect(actionTypeset, SIGNAL(triggered()), this, SLOT(interrupt()));
-		actionTypeset->setIcon(QIcon(":/images/images/runtool.png"));
+		actionTypeset->setIcon(QIcon(QString::fromLatin1(":/images/images/runtool.png")));
 		actionTypeset->setText(tr("Typeset"));
 		connect(actionTypeset, SIGNAL(triggered()), this, SLOT(retypeset()));
 	}
@@ -699,7 +822,7 @@ void PDFDocument::dragEnterEvent(QDragEnterEvent *event)
 	if (event->mimeData()->hasUrls()) {
 		const QList<QUrl> urls = event->mimeData()->urls();
 		foreach (const QUrl& url, urls) {
-			if (url.scheme() == "file") {
+			if (url.scheme() == QLatin1String("file")) {
 				event->acceptProposedAction();
 				break;
 			}
@@ -713,7 +836,7 @@ void PDFDocument::dropEvent(QDropEvent *event)
 	if (event->mimeData()->hasUrls()) {
 		const QList<QUrl> urls = event->mimeData()->urls();
 		foreach (const QUrl& url, urls)
-			if (url.scheme() == "file")
+			if (url.scheme() == QLatin1String("file"))
 				TWApp::instance()->openFile(url.toLocalFile());
 		event->acceptProposedAction();
 	}
@@ -724,6 +847,11 @@ void PDFDocument::contextMenuEvent(QContextMenuEvent *event)
 	Q_ASSERT(pdfWidget != NULL);
 	QMenu menu(this);
 
+	// Disarm the active tool (if any) as the menu will be highjacking all
+	// events. This will, e.g., close an open magnifier view, which would
+	// otherwise not receive a proper mouseReleaseEvent
+	pdfWidget->disarmTool();
+
 	if (_synchronizer && _synchronizer->isValid()) {
 		QAction *act = new QAction(tr("Jump to Source"), &menu);
 		act->setData(QVariant(event->pos()));
@@ -741,6 +869,11 @@ void PDFDocument::contextMenuEvent(QContextMenuEvent *event)
 	menu.exec(event->globalPos());
 }
 
+void PDFDocument::mouseMoveEvent(QMouseEvent *event)
+{
+	if (_fullScreenManager) _fullScreenManager->mouseMoveEvent(event);
+}
+
 void PDFDocument::jumpToSource()
 {
 	QAction *act = qobject_cast<QAction*>(sender());
@@ -771,12 +904,12 @@ void PDFDocument::doFindAgain(bool newSearch /* = false */)
 {
 	QSETTINGS_OBJECT(settings);
 
-	QString	searchText = settings.value("searchText").toString();
+	QString	searchText = settings.value(QString::fromLatin1("searchText")).toString();
 	if (searchText.isEmpty())
 		return;
 
 	QtPDF::Backend::SearchFlags searchFlags;
-	QTextDocument::FindFlags flags = (QTextDocument::FindFlags)settings.value("searchFlags").toInt();
+	QTextDocument::FindFlags flags = (QTextDocument::FindFlags)settings.value(QString::fromLatin1("searchFlags")).toInt();
 
 	if ((flags & QTextDocument::FindCaseSensitively) == 0)
 		searchFlags |= QtPDF::Backend::Search_CaseInsensitive;
@@ -786,7 +919,7 @@ void PDFDocument::doFindAgain(bool newSearch /* = false */)
 	widget()->search(searchText, searchFlags);
 }
 
-void PDFDocument::searchResultHighlighted(const int pageNum, const QList<QPolygonF> region)
+void PDFDocument::searchResultHighlighted(const int pageNum, const QList<QPolygonF> pdfRegion)
 {
 	QSETTINGS_OBJECT(settings);
 
@@ -794,19 +927,49 @@ void PDFDocument::searchResultHighlighted(const int pageNum, const QList<QPolygo
 	if (kPDFHighlightDuration > 0)
 		_searchResultHighlightRemover.start(kPDFHighlightDuration);
 
-	if (hasSyncData() && settings.value("searchPdfSync").toBool() && !region.isEmpty()) {
-		// emit a syncClick message at the center of the left edge of the (bounding)
-		// rect. To ensure hit-testing succeeds later on, we add an offset of 1e-5
-		// (for rectangles of finite width)
-		QRectF r = region[0].boundingRect();
-		QPointF pt(r.left() + 1e-5 * qMin(r.width(), 1.), r.center().y());
-		emit syncClick(pageNum, pt);
+	if (hasSyncData() && settings.value(QString::fromLatin1("searchPdfSync")).toBool() && !pdfRegion.isEmpty()) {
+		// In order to properly synchronize the search result, we first obtain
+		// the bounding boxes of the first and last matched characters and then
+		// use those for synchronization
+
+		QSharedPointer<QtPDF::Backend::Document> doc = widget()->document().toStrongRef();
+		Q_ASSERT(doc);
+		QSharedPointer<QtPDF::Backend::Page> page = doc->page(pageNum);
+		Q_ASSERT(page);
+		QMap<int, QRectF> charBoxes;
+
+		// NOTE: pdfRegion is in PDF coordinates (i.e., (0,0) is in the lower
+		// left), whereas QtPDF::Backend::Page::selectedText() expects TeX
+		// coordinates (i.e., (0,0) in the upper left). Hence we need to convert
+		// the coordinates
+		QList<QPolygonF> region;
+		foreach (QPolygonF pdfPolygon, pdfRegion) {
+			QPolygonF polygon;
+			foreach (QPointF p, pdfPolygon)
+				polygon << QPointF(p.x(), page->pageSizeF().height() - p.y());
+			region << polygon;
+		}
+
+		// Obtain the chracter bounding boxes of the search result
+		page->selectedText(region, NULL, &charBoxes, true);
+		Q_ASSERT(charBoxes.size() > 0);
+
+		// Obtain the centers of the first and last character bounding boxes and
+		// convert them to PDF coordinates (i.e., (0,0) in the lower left) as
+		// required by syncRange()
+		QPointF pt1 = charBoxes[0].center();
+		QPointF pt2 = charBoxes[charBoxes.size() - 1].center();
+		pt1.ry() = page->pageSizeF().height() - pt1.y();
+		pt2.ry() = page->pageSizeF().height() - pt2.y();
+
+		// Perform the synchronization
+		syncRange(pageNum, pt1, pt2, TWSynchronizer::CharacterResolution);
 	}
 }
 
 void PDFDocument::setDefaultScale() {
 	QSETTINGS_OBJECT(settings);
-	switch (settings.value("scaleOption", kDefault_PreviewScaleOption).toInt()) {
+	switch (settings.value(QString::fromLatin1("scaleOption"), kDefault_PreviewScaleOption).toInt()) {
 		case 2:
 			pdfWidget->zoomFitWidth();
 			break;
@@ -814,7 +977,7 @@ void PDFDocument::setDefaultScale() {
 			pdfWidget->zoomFitWindow();
 			break;
 		case 4:
-			pdfWidget->setZoomLevel(settings.value("previewScale", kDefault_PreviewScale).toFloat() / 100.);
+		    pdfWidget->setZoomLevel(settings.value(QString::fromLatin1("previewScale"), kDefault_PreviewScale).toFloat() / 100.);
 			break;
 		default:
 			pdfWidget->zoom100();
@@ -822,12 +985,38 @@ void PDFDocument::setDefaultScale() {
 	}
 }
 
+void PDFDocument::maybeOpenUrl(const QUrl url)
+{
+	// Opening URLs could be a security risk, so ask the user (but make "yes,
+	// proceed the default option - after all the user typically clicked on the
+	// link deliberately)
+	if (QMessageBox::question(this, tr("Open URL"), tr("You are in the process of opening the URL %1. Opening unknown or untrusted web adresses can be a security risk.\nDo you want to continue?").arg(url.toString()),
+	                          QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes)
+		QDesktopServices::openUrl(url);
+}
+
+void PDFDocument::maybeOpenPdf(QString filename, QtPDF::PDFDestination destination, bool newWindow)
+{
+	// Unlike in maybeOpenUrl, this function only works on local PDF files which
+	// we assume are safe.
+	// TODO: We currently ignore the value of newWindow and always open a new
+	// window. This avoids the need to update/invalidate all pointers to this
+	// PDFDocument (e.g., in the TeXDocument associated with it) to notify the
+	// other parts of the code that a completely new and unrelated document is
+	// loaded here now.
+	PDFDocument * pdf = qobject_cast<PDFDocument*>(TWApp::instance()->openFile(filename));
+	if (!pdf || !pdf->widget())
+		return;
+	pdf->widget()->goToPDFDestination(destination, false);
+}
+
+
 void PDFDocument::print()
 {
 	// Currently, printing is not supported in a reliable, cross-platform way
 	// Instead, offer to open the document in the system's default viewer
 	
-	QString msg = tr("Unfortunately, this version of %1 is unable to print Pdf documents due to various technical reasons.\n").arg(TEXWORKS_NAME);
+	QString msg = tr("Unfortunately, this version of %1 is unable to print Pdf documents due to various technical reasons.\n").arg(QString::fromLatin1(TEXWORKS_NAME));
 	msg += tr("Do you want to open the file in the default viewer for printing instead?");
 	msg += tr(" (remember to close it again to avoid access problems)");
 	
@@ -842,35 +1031,37 @@ void PDFDocument::print()
 void PDFDocument::showScaleContextMenu(const QPoint pos)
 {
 	static QMenu * contextMenu = NULL;
-	static QSignalMapper * contextMenuMapper = NULL;
-	QAction * a;
 	
 	if (contextMenu == NULL) {
 		contextMenu = new QMenu(this);
-		contextMenuMapper = new QSignalMapper(this);
+		static QSignalMapper * contextMenuMapper = new QSignalMapper(this);
+		QAction * a;
 		
 		contextMenu->addAction(actionFit_to_Width);
 		contextMenu->addAction(actionFit_to_Window);
 		contextMenu->addSeparator();
 		
-		a = contextMenu->addAction("200%");
+		a = contextMenu->addAction(tr("Custom..."));
+		connect(a, SIGNAL(triggered()), this, SLOT(doScaleDialog()));
+
+		a = contextMenu->addAction(tr("200%"));
 		connect(a, SIGNAL(triggered()), contextMenuMapper, SLOT(map()));
-		contextMenuMapper->setMapping(a, "2");
-		a = contextMenu->addAction("150%");
+		contextMenuMapper->setMapping(a, QString::fromLatin1("2"));
+		a = contextMenu->addAction(tr("150%"));
 		connect(a, SIGNAL(triggered()), contextMenuMapper, SLOT(map()));
-		contextMenuMapper->setMapping(a, "1.5");
+		contextMenuMapper->setMapping(a, QString::fromLatin1("1.5"));
 		// "100%" corresponds to "Actual Size", but we keep the numeric value
 		// here for consistency
-		a = contextMenu->addAction("100%");
+		a = contextMenu->addAction(tr("100%"));
 		a->setShortcut(actionActual_Size->shortcut());
 		connect(a, SIGNAL(triggered()), contextMenuMapper, SLOT(map()));
-		contextMenuMapper->setMapping(a, "1");
-		a = contextMenu->addAction("75%");
+		contextMenuMapper->setMapping(a, QString::fromLatin1("1"));
+		a = contextMenu->addAction(tr("75%"));
 		connect(a, SIGNAL(triggered()), contextMenuMapper, SLOT(map()));
-		contextMenuMapper->setMapping(a, ".75");
-		a = contextMenu->addAction("50%");
+		contextMenuMapper->setMapping(a, QString::fromLatin1(".75"));
+		a = contextMenu->addAction(tr("50%"));
 		connect(a, SIGNAL(triggered()), contextMenuMapper, SLOT(map()));
-		contextMenuMapper->setMapping(a, ".5");
+		contextMenuMapper->setMapping(a, QString::fromLatin1(".5"));
 		
 		connect(contextMenuMapper, SIGNAL(mapped(const QString&)), this, SLOT(setScaleFromContextMenu(const QString&)));
 	}
@@ -921,3 +1112,177 @@ void PDFDocument::doPageDialog()
 	if (ok)
 		pdfWidget->goToPage(pageNo - 1);
 }
+
+void PDFDocument::doScaleDialog()
+{
+	bool ok;
+	Q_ASSERT(pdfWidget != NULL);
+
+	double newScale = QInputDialog::getDouble(this, tr("Set Zoom"), tr("Zoom level:"), 100 * pdfWidget->zoomLevel(), 0, 2147483647, 0, &ok);
+	if (ok)
+		pdfWidget->setZoomLevel(newScale / 100);
+}
+
+FullscreenManager::FullscreenManager(QMainWindow * parent)
+	: _parent(parent)
+{
+	addShortcut(Qt::Key_Escape, SLOT(toggleFullScreen()));
+	_menuBarTimer.setSingleShot(true);
+	_menuBarTimer.setInterval(500);
+	connect(&_menuBarTimer, SIGNAL(timeout()), this, SLOT(showMenuBar()));
+}
+
+//virtual
+FullscreenManager::~FullscreenManager()
+{
+	foreach (const shortcut_info & sci, _shortcuts) {
+		if (sci.shortcut) delete sci.shortcut;
+	}
+}
+
+void FullscreenManager::setFullscreen(const bool fullscreen /* = true */)
+{
+	if (!_parent) return;
+	if (fullscreen == isFullscreen()) return;
+
+	if (fullscreen) {
+		// entering full-screen mode
+
+		// store the visibilities of important widgets such as the menu bar
+		// before hiding them (so we can restore them when exiting fullscreen
+		// mode)
+		_normalVisibility.clear();
+		if (_parent->menuBar()) {
+			_normalVisibility[_parent->menuBar()] = _parent->menuBar()->isVisible();
+			hideMenuBar();
+		}
+		if (_parent->statusBar()) {
+			_normalVisibility[_parent->statusBar()] = _parent->statusBar()->isVisible();
+			_parent->statusBar()->hide();
+		}
+		foreach (QToolBar * tb, _parent->findChildren<QToolBar*>()) {
+			_normalVisibility[tb] = tb->isVisible();
+			tb->hide();
+		}
+
+		_parent->showFullScreen();
+
+		// Enable custom shortcuts
+		foreach (const shortcut_info & sci, _shortcuts) {
+			// Skip shortcuts that are associated with a menu QAction; those are
+			// enabled/disabled when the menubar is hidden/shown
+			if (sci.action) continue;
+			sci.shortcut->setEnabled(fullscreen);
+		}
+	}
+	else if (!fullscreen) {
+		// exiting full-screen mode
+		// stop the timer, just in case
+		_menuBarTimer.stop();
+
+		_parent->showNormal();
+
+		// restore visibilities
+		foreach (QWidget * w, _normalVisibility.keys())
+			w->setVisible(_normalVisibility[w]);
+
+		// Disable custom shortcuts
+		foreach (const shortcut_info & sci, _shortcuts)
+			sci.shortcut->setEnabled(false);
+	}
+
+	// Enable/disable mouse tracking (which is required for getting mouse move
+	// events when no mouse button is pressed) to be able to show/hide the
+	// menu bar depending on the mouse position
+	_parent->setMouseTracking(fullscreen);
+
+	emit fullscreenChanged(fullscreen);
+}
+
+bool FullscreenManager::isFullscreen() const
+{
+	if (!_parent) return false;
+	return _parent->windowState().testFlag(Qt::WindowFullScreen);
+}
+
+void FullscreenManager::toggleFullscreen()
+{
+	setFullscreen(!isFullscreen());
+}
+
+void FullscreenManager::mouseMoveEvent(QMouseEvent * event)
+{
+	if (!_parent || !_parent->menuBar() || !isFullscreen()) return;
+	const int thresholdHeight = 10;
+
+	// The menu bar is shown when the mouse stays within thresholdHeight
+	// from the top of the screen for _menuBarTimer.interval()
+	// When the mouse moves in(to) that area and the timer is not running,
+	// start it
+	if (!_parent->menuBar()->isVisible()) {
+		if (event->pos().y() <= thresholdHeight && !_menuBarTimer.isActive()) _menuBarTimer.start();
+		// When the mouse moves out(side) of that area and the timer is running,
+		// stop it
+		else if (event->pos().y() > thresholdHeight && _menuBarTimer.isActive()) _menuBarTimer.stop();
+	}
+	// The menu bar is hidden whenever the mouse moves off of the menu bar
+	// (Note: when opening a menu, that menu intercepts all mouse events so we
+	// don't need to worry about the menu bar disappearing while the user
+	// browses through the menus.
+	else if (event->pos().y() > _parent->menuBar()->height())
+		hideMenuBar();
+}
+
+void FullscreenManager::addShortcut(QAction * action, const char * member)
+{
+	addShortcut(action->shortcut(), member, action);
+}
+
+void FullscreenManager::addShortcut(const QKeySequence & key, const char * member, QAction * action /* = NULL */)
+{
+	shortcut_info sci;
+	sci.shortcut = new QShortcut(key, _parent, member);
+	sci.shortcut->setEnabled(false);
+	sci.action = action;
+	if (action)
+		connect(action, SIGNAL(destroyed(QObject*)), this, SLOT(actionDeleted(QObject*)));
+	_shortcuts << sci;
+}
+
+void FullscreenManager::actionDeleted(QObject * obj)
+{
+	QAction * a = qobject_cast<QAction *>(obj);
+	if (!a) return;
+	for (unsigned int i = 0; i < _shortcuts.size(); ) {
+		if (_shortcuts[i].action == a) {
+			if (_shortcuts[i].shortcut)
+				delete _shortcuts[i].shortcut;
+			_shortcuts.removeAt(i);
+		}
+		else
+			++i;
+	}
+}
+
+void FullscreenManager::setMenuBarVisible(const bool visible /* = true */)
+{
+	if (!_parent || !_parent->menuBar()) return;
+	if (visible == _parent->menuBar()->isVisible()) return;
+
+	_parent->menuBar()->setVisible(visible);
+
+	// Enable our shortcut overrides when the menubar is hidden (to ensure that
+	// the most important shortcuts are available even when the corresponding
+	// QActions are hidden and disabled)
+	foreach (const shortcut_info & sci, _shortcuts) {
+		// Skip shortcuts that are not associated with any menu QAction
+		if (!sci.action) continue;
+
+		// If the shortcut gets enabled and corresponds to a valid, named
+		// QAction, update it to the QAction's key sequence (in case that
+		// got changed in the meantime)
+		if (!visible)
+			sci.shortcut->setKey(sci.action->shortcut());
+		sci.shortcut->setEnabled(!visible);
+	}
+}
